/*
 * Copyright (c) 2022-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#ifndef AM_COMMON_H_
#define AM_COMMON_H_

#include <iomanip>
#include <sstream>
#include <set>
#include <string>
#include <map>
#include <functional>
#include <type_traits>

#define AM_EXE_NAME "sharp_am"

class OptionManager;
struct OptionInfo;
class EventManager;
class Fabric;

extern bool g_should_update_options;
extern bool g_should_terminate;
extern OptionManager g_option_manager;
extern OptionInfo g_option_info;
extern EventManager g_event_manager;
extern Fabric g_fabric;

class SharpAggMgr {
    static int   ms_exit_code_;
    static bool  ms_stop_;

public:
    static void ExitAggMgr(int exit_code);
    static bool IsStopped() {return ms_stop_; }
    static void SetStop() {ms_stop_ = true; } //stop without sending signl
    static int GetExitCode() { return ms_exit_code_;}
};

class AMCommon {
    static long s_cache_line_size;

public:
    static void FlushAndCloseFile(FILE *f, const char *file_name);
    static void RetrieveCacheLineSize();    // Gets the value from the system
    static inline long GetCacheLineSize() { // Gets the saves value, need to be used after retrieving from the system
        return s_cache_line_size;
    }
    static int ParseGuidFile(std::set< uint64_t > &guids, std::string &guid_file);
    static bool FailoverEnabled();
};

template<typename EnumType>
bool ConvertStringToEnum(const std::string& field_str,
                         const std::map<std::string, EnumType>& str_to_enum_map,
                         EnumType& field)
{
    const auto map_it = str_to_enum_map.find(field_str);
    if (map_it == str_to_enum_map.end()) {
        return false;
    }
    field = map_it->second;
    return true;
};

namespace hash_utils {
template<typename ContainerType>
uint64_t XorHashCombine(const ContainerType& elements, 
                        std::function<uint64_t(const typename ContainerType::value_type&)> get_id_from_element_cb)
{
    uint64_t hash_result{0};
    for (const auto& current_element : elements) {
        hash_result ^= std::hash<uint64_t>{}(get_id_from_element_cb(current_element));
    }
    return hash_result;
}
}

template <typename NumberType, std::size_t FillSize = 16>
std::string NumberToHexStr(const NumberType num)
{
    std::stringstream ss{};
    ss << "0x" << std::setfill('0') << std::setw(FillSize) << std::hex << num;
    return ss.str();
}

uint64_t HexStrToNumber(const std::string& hex_str);

template <typename NumberType>
NumberType MegaBytesToBytes(const NumberType mb)
{
    return mb*1024*1024;
}

#endif //AM_COMMON_H_
